import struct

from umodbus.client.tcp import _create_request_adu, _create_mbap_header


def validate_mbap_fields(mbap, slave_id, pdu):
    """ Check if fields in MBAP header contain expected values. """
    transaction_id = struct.unpack('>H', mbap[:2])[0]
    protocol_id = struct.unpack('>H', mbap[2:4])[0]
    length = struct.unpack('>H', mbap[4:6])[0]
    unit_id = struct.unpack('>B', mbap[6:])[0]

    assert len(mbap) == 7
    assert 0 <= transaction_id <= 65536
    assert protocol_id == 0
    assert length == len(pdu) + 1
    assert unit_id == slave_id


def test_create_request_adu():
    """ Validate MBAP header of ADU and check if ADU contains correct PDU. """
    pdu = b'\x01'
    slave_id = 1
    adu = _create_request_adu(slave_id, pdu)

    # 9 is length MBAP (7 bytes) with length of PDU (1 byte)
    assert len(adu) == 8
    assert adu[7:] == pdu
    validate_mbap_fields(adu[:7], slave_id, pdu)


def test_create_mbap_header():
    """ Validate fields of MBAP header. """
    pdu = b'\x01x02'
    slave_id = 1
    mbap = _create_mbap_header(slave_id, pdu)

    validate_mbap_fields(mbap, slave_id, pdu)



try:
    from socketserver import BaseRequestHandler
except ImportError:
    from SocketServer import BaseRequestHandler
from binascii import hexlify

from umodbus import log
from umodbus.functions import create_function_from_request_pdu
from umodbus.exceptions import ModbusError, ServerDeviceFailureError
from umodbus.utils import (get_function_code_from_request_pdu,
                           pack_exception_pdu, recv_exactly)


def route(self, slave_ids=None, function_codes=None, addresses=None):
    """ A decorator that is used to register an endpoint for a given
    rule::

        @server.route(slave_ids=[1], function_codes=[1, 2], addresses=list(range(100, 200)))  # NOQA
        def read_single_bit_values(slave_id, address):
            return random.choise([0, 1])

    Any argument can be omitted to match any value.

    :param slave_ids: A list (or iterable) of slave ids.
    :param function_codes: A list (or iterable) of function codes.
    :param addresses: A list (or iterable) of addresses.
    """
    def inner(f):
        self.route_map.add_rule(f, slave_ids, function_codes, addresses)
        return f

    return inner


class AbstractRequestHandler(BaseRequestHandler):
    """ A subclass of :class:`socketserver.BaseRequestHandler` dispatching
    incoming Modbus requests using the server's :attr:`route_map`.

    """
    def handle(self):
        try:
            while True:
                try:
                    mbap_header = recv_exactly(self.request.recv, 7)
                    remaining = self.get_meta_data(mbap_header)['length'] - 1
                    request_pdu = recv_exactly(self.request.recv, remaining)
                except ValueError:
                    return

                response_adu = self.process(mbap_header + request_pdu)
                self.respond(response_adu)
        except:
            log.exception('Error while handling request')
            raise

    def process(self, request_adu):
        """ Process request ADU and return response.

        :param request_adu: A bytearray containing the ADU request.
        :return: A bytearray containing the response of the ADU request.
        """
        meta_data = self.get_meta_data(request_adu)
        request_pdu = self.get_request_pdu(request_adu)

        response_pdu = self.execute_route(meta_data, request_pdu)
        response_adu = self.create_response_adu(meta_data, response_pdu)

        return response_adu

    def execute_route(self, meta_data, request_pdu):
        """ Execute configured route based on requests meta data and request
        PDU.

        :param meta_data: A dict with meta data. It must at least contain
            key 'unit_id'.
        :param request_pdu: A bytearray containing request PDU.
        :return: A bytearry containing reponse PDU.
        """
        try:
            function = create_function_from_request_pdu(request_pdu)
            results =\
                function.execute(meta_data['unit_id'], self.server.route_map)

            try:
                # ReadFunction's use results of callbacks to build response
                # PDU...
                return function.create_response_pdu(results)
            except TypeError:
                # ...other functions don't.
                return function.create_response_pdu()
        except ModbusError as e:
            function_code = get_function_code_from_request_pdu(request_pdu)
            return pack_exception_pdu(function_code, e.error_code)
        except Exception:
            log.exception('Could not handle request')
            function_code = get_function_code_from_request_pdu(request_pdu)

            return pack_exception_pdu(function_code,
                                      ServerDeviceFailureError.error_code)

    def respond(self, response_adu):
        """ Send response ADU back to client.

        :param response_adu: A bytearray containing the response of an ADU.
        """
        log.debug('--> {0} - {1}.'.format(self.client_address[0],
                  hexlify(response_adu)))
        self.request.sendall(response_adu)



import struct
from types import MethodType

from umodbus.route import Map
from umodbus.server import AbstractRequestHandler, route
from umodbus.utils import unpack_mbap, pack_mbap
from umodbus.exceptions import ServerDeviceFailureError


def get_server(server_class, server_address, request_handler_class):
    """ Return instance of :param:`server_class` with :param:`request_handler`
    bound to it.
    This method also binds a :func:`route` method to the server instance.
        >>> server = get_server(TcpServer, ('localhost', 502), RequestHandler)
        >>> server.serve_forever()
    :param server_class: (sub)Class of :class:`socketserver.BaseServer`.
    :param request_handler_class: (sub)Class of
        :class:`umodbus.server.RequestHandler`.
    :return: Instance of :param:`server_class`.
    """
    s = server_class(server_address, request_handler_class)

    s.route_map = Map()
    s.route = MethodType(route, s)

    return s


class RequestHandler(AbstractRequestHandler):
    """ A subclass of :class:`socketserver.BaseRequestHandler` dispatching
    incoming Modbus TCP/IP request using the server's :attr:`route_map`.

    """
    def get_meta_data(self, request_adu):
        """" Extract MBAP header from request adu and return it. The dict has
        4 keys: transaction_id, protocol_id, length and unit_id.

        :param request_adu: A bytearray containing request ADU.
        :return: Dict with meta data of request.
        """
        try:
            transaction_id, protocol_id, length, unit_id = \
                unpack_mbap(request_adu[:7])
        except struct.error:
            raise ServerDeviceFailureError()

        return {
            'transaction_id': transaction_id,
            'protocol_id': protocol_id,
            'length': length,
            'unit_id': unit_id,
        }

    def get_request_pdu(self, request_adu):
        """ Extract PDU from request ADU and return it.

        :param request_adu: A bytearray containing request ADU.
        :return: An bytearray container request PDU.
        """
        return request_adu[7:]

    def create_response_adu(self, meta_data, response_pdu):
        """ Build response ADU from meta data and response PDU and return it.

        :param meta_data: A dict with meta data.
        :param request_pdu: A bytearray containing request PDU.
        :return: A bytearray containing request ADU.
        """
        response_mbap = pack_mbap(
            transaction_id=meta_data['transaction_id'],
            protocol_id=meta_data['protocol_id'],
            length=len(response_pdu) + 1,
            unit_id=meta_data['unit_id']
        )

        return response_mbap + response_pdu



#!/usr/bin/env python
# scripts/examples/simple_data_store.py
import logging
from socketserver import TCPServer
from collections import defaultdict

from umodbus import conf
from umodbus.server.tcp import RequestHandler, get_server
from umodbus.utils import log_to_stream

# Add stream handler to logger 'uModbus'.
log_to_stream(level=logging.DEBUG)

# A very simple data store which maps addresses against their values.
data_store = defaultdict(int)

# Enable values to be signed (default is False).
conf.SIGNED_VALUES = True

TCPServer.allow_reuse_address = True
app = get_server(TCPServer, ('localhost', 502), RequestHandler)


@app.route(slave_ids=[1], function_codes=[1, 2], addresses=list(range(0, 10)))
def read_data_store(slave_id, function_code, address):
    """" Return value of address. """
    return data_store[address]


@app.route(slave_ids=[1], function_codes=[5, 15], addresses=list(range(0, 10)))
def write_data_store(slave_id, function_code, address, value):
    """" Set value for address. """
    data_store[address] = value

if __name__ == '__main__':
    try:
        app.serve_forever()
    finally:
        app.shutdown()
        app.server_close()



#!/usr/bin/env python
from serial import Serial
from collections import defaultdict

from umodbus.server.serial import get_server
from umodbus.server.serial.rtu import RTUServer

s = Serial('/dev/ttyS1')
s.timeout = 10

data_store = defaultdict(int)
app = get_server(RTUServer, s)


@app.route(slave_ids=[1], function_codes=[1, 2], addresses=list(range(0, 10)))
def read_data_store(slave_id, function_code, address):
    """" Return value of address. """
    return data_store[address]


@app.route(slave_ids=[1], function_codes=[5, 15], addresses=list(range(0, 10)))
def write_data_store(slave_id, function_code, address, value):
    """" Set value for address. """
    data_store[address] = value

if __name__ == '__main__':
    try:
        app.serve_forever()
    finally:
        app.shutdown()



import struct
import pytest

try:
    # Mock has been added to stdlib in Python 3.3.
    from unittest.mock import MagicMock, call
except ImportError:
    from mock import MagicMock, call

from umodbus.route import Map
from umodbus.exceptions import (IllegalFunctionError, IllegalDataAddressError,
                                IllegalDataValueError,
                                ServerDeviceFailureError, AcknowledgeError,
                                ServerDeviceBusyError, MemoryParityError,
                                GatewayPathUnavailableError,
                                GatewayTargetDeviceFailedToRespondError)
from umodbus.functions import (create_function_from_response_pdu,
                               create_function_from_request_pdu, ReadCoils,
                               ReadDiscreteInputs, ReadHoldingRegisters,
                               ReadInputRegisters, WriteSingleCoil,
                               WriteSingleRegister, WriteMultipleCoils,
                               WriteMultipleRegisters)


@pytest.fixture
def read_coils():
    instance = ReadCoils()
    instance.starting_address = 100
    instance.quantity = 3

    return instance


@pytest.fixture
def read_discrete_inputs():
    instance = ReadDiscreteInputs()
    instance.starting_address = 0
    instance.quantity = 2

    return instance


@pytest.fixture
def read_holding_registers():
    instance = ReadHoldingRegisters()
    instance.starting_address = 100
    instance.quantity = 4

    return instance


@pytest.fixture
def read_input_registers():
    instance = ReadInputRegisters()
    instance.starting_address = 50
    instance.quantity = 2

    return instance


@pytest.fixture
def write_single_coil():
    instance = WriteSingleCoil()
    instance.address = 100
    instance.value = 1

    return instance


@pytest.fixture
def write_single_register():
    instance = WriteSingleRegister()
    instance.address = 200
    instance.value = 18

    return instance


@pytest.fixture
def write_multiple_coils():
    instance = WriteMultipleCoils()
    instance.starting_address = 100
    instance.values = [1, 0]

    return instance


@pytest.fixture
def write_multiple_registers():
    instance = WriteMultipleRegisters()
    instance.starting_address = 50
    instance.values = [1337, 15, 128]

    return instance


@pytest.fixture
def route_map():
    return Map()


@pytest.mark.parametrize('pdu,cls', [
    (b'\x01\x00d\x00\x03', ReadCoils),
    (b'\x02\x00d\x00\x03', ReadDiscreteInputs),
    (b'\x03\x00d\x00\x03', ReadHoldingRegisters),
    (b'\x04\x00d\x00\x03', ReadInputRegisters),
    (b'\x05\x00d\x00\x00', WriteSingleCoil),
    (b'\x06\x00d\x00\x00', WriteSingleRegister),
    (b'\x0f\x00d\x00\x03\x01\x04', WriteMultipleCoils),
    (b'\x10\x00d\x00\x01\x02\x00\x04', WriteMultipleRegisters),
])
def test_create_function_from_request_pdu(pdu, cls):
    assert isinstance(create_function_from_request_pdu(pdu), cls)


@pytest.mark.parametrize('error_code, exception_class', [
    (1, IllegalFunctionError),
    (2, IllegalDataAddressError),
    (3, IllegalDataValueError),
    (4, ServerDeviceFailureError),
    (5, AcknowledgeError),
    (6, ServerDeviceBusyError),
    (8, MemoryParityError),
    (10, GatewayPathUnavailableError),
    (11, GatewayTargetDeviceFailedToRespondError),
])
def test_create_from_response_pdu_raising_exception(error_code,
                                                    exception_class):
    """ Test if correct exception is raied when response PDU contains error
    response.
    """
    resp_pdu = struct.pack('>BB', 81, error_code)

    with pytest.raises(exception_class):
        create_function_from_response_pdu(resp_pdu)


def test_create_function_from_request_pdu_raising_illegal_function_error():
    """ Calling function with PDU containing invalid function code should result
    in an IllegalFunctionError.
    """
    with pytest.raises(IllegalFunctionError):
        create_function_from_request_pdu(b'\x00')


def test_read_coils_class_attributes():
    assert ReadCoils.function_code == 1
    assert ReadCoils.max_quantity == 2000


def test_read_coils_with_invalid_attributes(read_coils):
    with pytest.raises(IllegalDataValueError):
        read_coils.quantity = 2001


def test_read_coils_request_pdu(read_coils):
    instance = ReadCoils.create_from_request_pdu(read_coils.request_pdu)
    assert instance.starting_address == 100
    assert instance.quantity == 3


def test_read_coils_response_pdu(read_coils):
    response_pdu = read_coils.create_response_pdu([0, 1, 1])
    instance = ReadCoils.create_from_response_pdu(response_pdu, read_coils.request_pdu)  # NOQA

    assert instance.data == [0, 1, 1]


def test_read_discrete_inputs_class_attributes():
    assert ReadDiscreteInputs.function_code == 2
    assert ReadDiscreteInputs.max_quantity == 2000


def test_read_discrete_inputs_with_invalid_attributes(read_discrete_inputs):
    with pytest.raises(IllegalDataValueError):
        read_discrete_inputs.quantity = 2001


def test_read_discrete_inputs_request_pdu(read_discrete_inputs):
    instance = ReadDiscreteInputs.create_from_request_pdu(read_discrete_inputs.request_pdu)  # NOQA
    assert instance.starting_address == 0
    assert instance.quantity == 2


def test_read_discrete_inputs_response_pdu(read_discrete_inputs):
    response_pdu = read_discrete_inputs.create_response_pdu([1, 0])
    instance = ReadDiscreteInputs.create_from_response_pdu(response_pdu, read_discrete_inputs.request_pdu)  # NOQA

    assert instance.data == [1, 0]


def test_read_holding_registers_class_attributes():
    assert ReadHoldingRegisters.function_code == 3
    assert ReadHoldingRegisters.max_quantity == 125


def test_read_holding_registers_with_invalid_attributes(read_holding_registers):  # NOQA
    with pytest.raises(IllegalDataValueError):
        read_holding_registers.quantity = 126


def test_read_holding_registers_request_pdu(read_holding_registers):
    instance = ReadHoldingRegisters.create_from_request_pdu(read_holding_registers.request_pdu)  # NOQA

    assert instance.starting_address == 100
    assert instance.quantity == 4


def test_read_holding_registers_response_pdu(read_holding_registers):
    response_pdu =\
        read_holding_registers.create_response_pdu([1337, 17, 21, 18])

    instance = ReadHoldingRegisters.create_from_response_pdu(response_pdu, read_holding_registers.request_pdu)  # NOQA

    assert instance.data == [1337, 17, 21, 18]


def test_read_input_registers_class_attributes():
    assert ReadInputRegisters.function_code == 4
    assert ReadInputRegisters.max_quantity == 125


def test_read_input_registers_with_invalid_attributes(read_input_registers):  # NOQA
    with pytest.raises(IllegalDataValueError):
        read_input_registers.quantity = 126


def test_read_input_registers_request_pdu(read_input_registers):
    instance = ReadInputRegisters.create_from_request_pdu(read_input_registers.request_pdu)  # NOQA

    assert instance.starting_address == 50
    assert instance.quantity == 2


def test_read_input_registers_response_pdu(read_input_registers):
    response_pdu = read_input_registers.create_response_pdu([994, 1100])
    instance = ReadInputRegisters.create_from_response_pdu(response_pdu, read_input_registers.request_pdu)  # NOQA

    assert instance.data == [994, 1100]


def test_write_single_coil_class_attributes():
    assert WriteSingleCoil.function_code == 5


def test_write_single_coil_with_invalid_attributes(write_single_coil):
    with pytest.raises(IllegalDataValueError):
        write_single_coil.value = 5


def test_write_single_coil_request_pdu(write_single_coil):
    instance = WriteSingleCoil.create_from_request_pdu(write_single_coil.request_pdu)  # NOQA

    assert instance.address == 100
    assert instance.value == 1


def test_write_single_coil_response_pdu(write_single_coil):
    response_pdu = write_single_coil.create_response_pdu()
    instance = WriteSingleCoil.create_from_response_pdu(response_pdu)

    assert instance.address == 100
    assert instance.data == 1


def test_write_single_registers_class_attributes():
    assert WriteSingleRegister.function_code == 6


def test_write_single_register_with_invalid_attributes(write_single_register):
    with pytest.raises(IllegalDataValueError):
        write_single_register.value = -1


def test_write_single_register_request_pdu(write_single_register):
    instance = WriteSingleRegister.create_from_request_pdu(write_single_register.request_pdu)  # NOQA

    assert instance.address == 200
    assert instance.value == 18


def test_write_single_register_response_pdu(write_single_register):
    response_pdu = write_single_register.create_response_pdu()
    instance = WriteSingleRegister.create_from_response_pdu(response_pdu)

    assert instance.address == 200
    assert instance.data == 18


def test_write_multiple_coils_class_attributes():
    WriteMultipleCoils.function_code == 15


def test_write_multiple_coils_invalid_attributes(write_multiple_coils):
    with pytest.raises(IllegalDataValueError):
        write_multiple_coils.values = []

    with pytest.raises(IllegalDataValueError):
        write_multiple_coils.values = [5]


def test_write_multiple_coils_request_pdu(write_multiple_coils):
    instance = WriteMultipleCoils.create_from_request_pdu(write_multiple_coils.request_pdu)  # NOQA

    assert instance.starting_address == 100
    assert instance.values == [1, 0]


def test_write_multiple_coils_response_pdu(write_multiple_coils):
    response_pdu = write_multiple_coils.create_response_pdu()
    instance = WriteMultipleCoils.create_from_response_pdu(response_pdu)

    assert instance.starting_address == 100
    assert instance.data == 2


def test_write_multiple_registers_class_attributes():
    WriteMultipleRegisters.function_code == 16


def test_write_multiple_registers_invalid_attributes(write_multiple_registers):
    with pytest.raises(IllegalDataValueError):
        write_multiple_registers.values = []

    with pytest.raises(IllegalDataValueError):
        write_multiple_registers.values = [-1]


def test_write_multiple_registers_request_pdu(write_multiple_registers):
    instance = WriteMultipleRegisters.create_from_request_pdu(write_multiple_registers.request_pdu)  # NOQA

    assert instance.starting_address == 50
    assert instance.values == [1337, 15, 128]


def test_write_multiple_registers_response_pdu(write_multiple_registers):
    response_pdu = write_multiple_registers.create_response_pdu()
    instance = WriteMultipleRegisters.create_from_response_pdu(response_pdu)

    assert instance.starting_address == 50
    assert instance.data == 3


def test_create_function_from_response_pdu():
    read_coils = ReadCoils()
    read_coils.starting_address = 1
    read_coils.quantity = 9

    req_pdu = read_coils.request_pdu
    resp_pdu = struct.pack('>BBB', 1, 1, 3)

    assert isinstance(create_function_from_response_pdu(resp_pdu, req_pdu),
                      ReadCoils)



def bind_routes(server):
    server.route_map.add_rule(read_status, slave_ids=[1], function_codes=[1, 2], addresses=list(range(0, 10)))  # NOQA
    server.route_map.add_rule(read_register, slave_ids=[1], function_codes=[3, 4], addresses=list(range(0, 10)))  # NOQA
    server.route_map.add_rule(write_status, slave_ids=[1], function_codes=[5, 15], addresses=list(range(0, 10)))  # NOQA
    server.route_map.add_rule(write_register, slave_ids=[1], function_codes=[6, 16], addresses=list(range(0, 10)))  # NOQA
    server.route_map.add_rule(failure, slave_ids=[1], function_codes=[1, 2, 3, 4, 5, 6, 15, 16], addresses=[666])  # NOQA


def read_status(slave_id, function_code, address):
    return address % 2


def read_register(slave_id, function_code, address):
    return -address


def write_status(slave_id, function_code, address, value):
    pass


def write_register(slave_id, function_code, address, value):
    pass


def failure(*args, **kwargs):
    raise Exception



import pytest

from umodbus import conf
from umodbus.config import Config


@pytest.fixture(scope='module', autouse=True)
def enable_signed_values(request):
    """ Use signed values when running tests it this module. """
    tmp = conf.SIGNED_VALUES
    conf.SIGNED_VALUES = False

    def fin():
        conf.SIGNED_VALUES = tmp

    request.addfinalizer(fin)


@pytest.fixture
def config():
    return Config()



from serial import serial_for_url

from umodbus import conf
from umodbus.server.serial import get_server
from umodbus.server.serial.rtu import RTUServer

from tests.system import route

conf.SIGNED_VALUES = True

s = serial_for_url('loop://')
app = get_server(RTUServer, s)

route.bind_routes(app)



import pytest

from umodbus.server.serial import AbstractSerialServer


@pytest.fixture
def abstract_serial_server():
    return AbstractSerialServer()


def test_abstract_serial_server_get_meta_data(abstract_serial_server):
    """ Test if meta data is correctly extracted from request. """
    assert abstract_serial_server.get_meta_data(b'\x01x\02\x03') ==\
        {'unit_id': 1}


def test_abract_serial_server_shutdown(abstract_serial_server):
    assert abstract_serial_server._shutdown_request is False

    abstract_serial_server.shutdown()

    assert abstract_serial_server._shutdown_request is True



"""

.. note:: This section is based on `MODBUS Messaging on TCP/IP
    Implementation Guide V1.0b`_.

The Application Data Unit (ADU) for Modbus messages carried over a TCP/IP are
build out of two components: a MBAP header and a PDU. The Modbus Application
Header (MBAP) is what makes Modbus TCP/IP requests and responses different from
their counterparts send over a serial line.  Below the components of the Modbus
TCP/IP are listed together with their size in bytes:

+---------------+-----------------+
| **Component** | **Size** (bytes)|
+---------------+-----------------+
| MBAP Header   | 7               |
+---------------+-----------------+
| PDU           | N               |
+---------------+-----------------+

Below you see a hexadecimal presentation of request over TCP/IP with Modbus
function code 1. It requests data of slave with 1, starting at coil 100, for
the length of 3 coils:

..
    Note: the backslash in the bytes below are escaped using an extra back
    slash. Without escaping the bytes aren't printed correctly in the HTML
    output of this docs.

    To work with the bytes in Python you need to remove the escape sequences.
    `b'\\x01\\x00d` -> `b\x01\x00d`

.. code-block:: python

    >>> # Read coils, starting from coil 100 for the length of 3 coils.
    >>> adu = b'\\x00\\x08\\x00\\x00\\x00\\x06\\x01\\x01\\x00d\\x00\\x03'

The length of the ADU is 12 bytes::

    >>> len(adu)
    12

The MBAP header is 7 bytes long::

    >>> mbap = adu[:7]
    >>> mbap
    b'\\x00\\x08\\x00\\x00\\x00\\x06\\x01'

The MBAP header contains the following fields:

+------------------------+--------------------+--------------------------------------+
| **Field**              | **Length** (bytes) | **Description**                      |
+------------------------+--------------------+--------------------------------------+
| Transaction identifier | 2                  | Identification of a                  |
|                        |                    | Modbus request/response transaction. |
+------------------------+--------------------+--------------------------------------+
| Protocol identifier    | 2                  | Protocol ID, is 0 for Modbus.        |
+------------------------+--------------------+--------------------------------------+
| Length                 | 2                  | Number of following bytes            |
+------------------------+--------------------+--------------------------------------+
| Unit identifier        | 1                  | Identification of a                  |
|                        |                    | remote slave                         |
+------------------------+--------------------+--------------------------------------+

When unpacked, these fields have the following values::

    >>> transaction_id = mbap[:2]
    >>> transaction_id
    b'\\x00\\x08'
    >>> protocol_id = mbap[2:4]
    >>> protocol_id
    b'\\x00\\x00'
    >>> length = mbap[4:6]
    >>> length
    b'\\x00\\x06'
    >>> unit_id = mbap[6:]
    >>> unit_id
    b'\\0x01'

The request in words: a request with Transaction ID 8 for slave 1. The
request uses Protocol ID 0, which is the Modbus protocol. The length of the
bytes after the Length field is 6 bytes. These 6 bytes are Unit Identifier (1
byte) + PDU (5 bytes).

"""
import struct
from random import randint

from umodbus.functions import (create_function_from_response_pdu,
                               expected_response_pdu_size_from_request_pdu,
                               pdu_to_function_code_or_raise_error, ReadCoils,
                               ReadDiscreteInputs, ReadHoldingRegisters,
                               ReadInputRegisters, WriteSingleCoil,
                               WriteSingleRegister, WriteMultipleCoils,
                               WriteMultipleRegisters)
from umodbus.utils import recv_exactly


def _create_request_adu(slave_id, pdu):
    """ Create MBAP header and combine it with PDU to return ADU.

    :param slave_id: Number of slave.
    :param pdu: Byte array with PDU.
    :return: Byte array with ADU.
    """
    return _create_mbap_header(slave_id, pdu) + pdu


def _create_mbap_header(slave_id, pdu):
    """ Return byte array with MBAP header for PDU.

    :param slave_id: Number of slave.
    :param pdu: Byte array with PDU.
    :return: Byte array of 7 bytes with MBAP header.
    """
    # 65535 = (2**16)-1 aka maximum number that fits in 2 bytes.
    transaction_id = randint(0, 65535)
    length = len(pdu) + 1

    return struct.pack('>HHHB', transaction_id, 0, length, slave_id)


def read_coils(slave_id, starting_address, quantity):
    """ Return ADU for Modbus function code 01: Read Coils.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = ReadCoils()
    function.starting_address = starting_address
    function.quantity = quantity

    return _create_request_adu(slave_id, function.request_pdu)


def read_discrete_inputs(slave_id, starting_address, quantity):
    """ Return ADU for Modbus function code 02: Read Discrete Inputs.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = ReadDiscreteInputs()
    function.starting_address = starting_address
    function.quantity = quantity

    return _create_request_adu(slave_id, function.request_pdu)


def read_holding_registers(slave_id, starting_address, quantity):
    """ Return ADU for Modbus function code 03: Read Holding Registers.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = ReadHoldingRegisters()
    function.starting_address = starting_address
    function.quantity = quantity

    return _create_request_adu(slave_id, function.request_pdu)


def read_input_registers(slave_id, starting_address, quantity):
    """ Return ADU for Modbus function code 04: Read Input Registers.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = ReadInputRegisters()
    function.starting_address = starting_address
    function.quantity = quantity

    return _create_request_adu(slave_id, function.request_pdu)


def write_single_coil(slave_id, address, value):
    """ Return ADU for Modbus function code 05: Write Single Coil.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = WriteSingleCoil()
    function.address = address
    function.value = value

    return _create_request_adu(slave_id, function.request_pdu)


def write_single_register(slave_id, address, value):
    """ Return ADU for Modbus function code 06: Write Single Register.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = WriteSingleRegister()
    function.address = address
    function.value = value

    return _create_request_adu(slave_id, function.request_pdu)


def write_multiple_coils(slave_id, starting_address, values):
    """ Return ADU for Modbus function code 15: Write Multiple Coils.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = WriteMultipleCoils()
    function.starting_address = starting_address
    function.values = values

    return _create_request_adu(slave_id, function.request_pdu)


def write_multiple_registers(slave_id, starting_address, values):
    """ Return ADU for Modbus function code 16: Write Multiple Registers.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = WriteMultipleRegisters()
    function.starting_address = starting_address
    function.values = values

    return _create_request_adu(slave_id, function.request_pdu)


def parse_response_adu(resp_adu, req_adu=None):
    """ Parse response ADU and return response data. Some functions require
    request ADU to fully understand request ADU.

    :param resp_adu: Resonse ADU.
    :param req_adu: Request ADU, default None.
    :return: Response data.
    """
    resp_pdu = resp_adu[7:]
    function = create_function_from_response_pdu(resp_pdu, req_adu)

    return function.data


def raise_for_exception_adu(resp_adu):
    """ Check a response ADU for error

    :param resp_adu: Response ADU.
    :raises ModbusError: When a response contains an error code.
    """
    resp_pdu = resp_adu[7:]
    pdu_to_function_code_or_raise_error(resp_pdu)


def send_message(adu, sock):
    """ Send ADU over socket to to server and return parsed response.

    :param adu: Request ADU.
    :param sock: Socket instance.
    :return: Parsed response from server.
    """
    sock.sendall(adu)

    # Check exception ADU (which is shorter than all other responses) first.
    exception_adu_size = 9
    response_error_adu = recv_exactly(sock.recv, exception_adu_size)
    raise_for_exception_adu(response_error_adu)

    expected_response_size = \
        expected_response_pdu_size_from_request_pdu(adu[7:]) + 7
    response_remainder = recv_exactly(
        sock.recv, expected_response_size - exception_adu_size)

    return parse_response_adu(response_error_adu + response_remainder, adu)



import sys
import logging
from logging import getLogger

from umodbus.utils import (log_to_stream, unpack_mbap, pack_mbap,
                           pack_exception_pdu,
                           get_function_code_from_request_pdu)


def test_log_to_stream():
    """ Test if handler is added correctly. """
    log = getLogger('uModbus')

    # NullHandler is attached.
    assert len(log.handlers) == 1
    log_to_stream()
    assert len(log.handlers) == 2

    handler = log.handlers[1]
    assert handler.stream == sys.stderr
    assert handler.level == logging.NOTSET


def test_unpack_mbap():
    """ MBAP should contain correct values for Transaction identifier, Protocol
    identifier, Length and Unit identifer.
    """
    assert unpack_mbap(b'\x00\x08\x00\x00\x00\x06\x01') == (8, 0, 6, 1)


def test_pack_mbap():
    """ Byte array should contain correct encoding of Transaction identifier,
    Protocol identifier, Length and Unit identifier.
    """
    assert pack_mbap(8, 0, 6, 1) == b'\x00\x08\x00\x00\x00\x06\x01'


def test_pack_exception_pdu():
    """ Exception PDU should correct encoding of error code and function code.
    """
    assert pack_exception_pdu(1, 1) == b'\x81\x01'


def test_get_function_code_from_request_pdu():
    """ Get correct function code from PDU. """
    assert get_function_code_from_request_pdu(b'\x01\x00d\x00\x03') == 1



""" CRC is calculated over slave id + PDU.

Most code is taken from: https://github.com/pyhys/minimalmodbus/blob/e99f4d74c83258c6039073082955ac9bed3f2155/minimalmodbus.py  # NOQA
"""
import struct


def generate_look_up_table():
    """ Generate look up table.

    :return: List
    """
    poly = 0xA001
    table = []

    for index in range(256):

        data = index << 1
        crc = 0
        for _ in range(8, 0, -1):
            data >>= 1
            if (data ^ crc) & 0x0001:
                crc = (crc >> 1) ^ poly
            else:
                crc >>= 1
        table.append(crc)

    return table


look_up_table = generate_look_up_table()


def get_crc(msg):
    """ Return CRC of 2 byte for message.

        >>> assert get_crc(b'\x02\x07') == struct.unpack('<H', b'\x41\x12')

    :param msg: A byte array.
    :return: Byte array of 2 bytes.
    """
    register = 0xFFFF

    for byte_ in msg:
        try:
            val = struct.unpack('<B', byte_)[0]
        # Iterating over a bit-like objects in Python 3 gets you ints.
        # Because fuck logic.
        except TypeError:
            val = byte_

        register = \
            (register >> 8) ^ look_up_table[(register ^ val) & 0xFF]

    # CRC is little-endian!
    return struct.pack('<H', register)


def add_crc(msg):
    """ Append CRC to message.

    :param msg: A byte array.
    :return: Byte array.
    """
    return msg + get_crc(msg)


def validate_crc(msg):
    """ Validate CRC of message.

    :param msg: Byte array with message with CRC.
    :raise: CRCError.
    """
    if not struct.unpack('<H', get_crc(msg[:-2])) ==\
            struct.unpack('<H', msg[-2:]):
        raise CRCError('CRC validation failed.')


class CRCError(Exception):
    """ Valid error to raise when CRC isn't correct. """
    pass



import pytest
import struct
from functools import partial

from ..validators import validate_response_mbap
from umodbus.client import tcp


@pytest.mark.parametrize('function_code, quantity', [
    (1, 0),
    (2, 0),
    (3, 0),
    (4, 0),
    (1, 0x07D0 + 1),
    (2, 0x07D0 + 1),
    (3, 0x007D + 1),
    (4, 0x007D + 1),
])
def test_request_returning_invalid_data_value_error(sock, mbap, function_code,
                                                    quantity):
    """ Validate response PDU of request returning excepetion response with
    error code 3.
    """
    function_code, starting_address, quantity = (function_code, 0, quantity)
    adu = mbap + struct.pack('>BHH', function_code, starting_address, quantity)

    sock.send(adu)
    resp = sock.recv(1024)

    validate_response_mbap(mbap, resp)
    assert struct.unpack('>BB', resp[-2:]) == (0x80 + function_code, 3)


@pytest.mark.parametrize('function', [
    (partial(tcp.read_coils, 1, 9, 2)),
    (partial(tcp.read_discrete_inputs, 1, 9, 2)),
    (partial(tcp.read_holding_registers, 1, 9, 2)),
    (partial(tcp.read_input_registers, 1, 9, 2)),
    (partial(tcp.write_single_coil, 1, 11, 0)),
    (partial(tcp.write_single_register, 1, 11, 1337)),
    (partial(tcp.write_multiple_coils, 1, 9, [1, 1])),
    (partial(tcp.write_multiple_registers, 1, 9, [1337, 15])),
])
def test_request_returning_invalid_data_address_error(sock, function):
    """ Validate response PDU of request returning excepetion response with
    error code 2.
    """
    adu = function()

    mbap = adu[:7]
    function_code = struct.unpack('>B', adu[7:8])[0]

    sock.send(adu)
    resp = sock.recv(1024)

    validate_response_mbap(mbap, resp)
    assert struct.unpack('>BB', resp[-2:]) == (0x80 + function_code, 2)


@pytest.mark.parametrize('function', [
    (partial(tcp.read_coils, 1, 666, 1)),
    (partial(tcp.read_discrete_inputs, 1, 666, 1)),
    (partial(tcp.read_holding_registers, 1, 666, 1)),
    (partial(tcp.read_input_registers, 1, 666, 1)),
    (partial(tcp.write_single_coil, 1, 666, 0)),
    (partial(tcp.write_single_register, 1, 666, 1337)),
    (partial(tcp.write_multiple_coils, 1, 666, [1])),
    (partial(tcp.write_multiple_registers, 1, 666, [1337])),
])
def test_request_returning_server_device_failure_error(sock, function):
    """ Validate response PDU of request returning excepetion response with
    error code 4.
    """
    adu = function()

    mbap = adu[:7]
    function_code = struct.unpack('>B', adu[7:8])[0]

    sock.send(adu)
    resp = sock.recv(1024)

    validate_response_mbap(mbap, resp)
    assert struct.unpack('>BB', resp[-2:]) == (0x80 + function_code, 4)



import struct
import pytest
import socket
from threading import Thread

from .tcp_server import app as tcp
from .rtu_server import app as rtu


@pytest.fixture(autouse=True, scope="session")
def tcp_server(request):
    t = Thread(target=tcp.serve_forever)
    t.start()

    def fin():
        tcp.shutdown()
        tcp.server_close()
        t.join()

    request.addfinalizer(fin)
    return tcp


@pytest.yield_fixture
def sock(tcp_server):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect(tcp_server.socket.getsockname())

    yield sock

    sock.close()


@pytest.fixture
def rtu_server():
    return rtu


@pytest.fixture
def mbap():
    return struct.pack('>HHHB', 0, 0, 6, 1)



try:
    from socketserver import TCPServer
except ImportError:
    from SocketServer import TCPServer

from umodbus import conf
from umodbus.server.tcp import get_server, RequestHandler

from tests.system import route

conf.SIGNED_VALUES = True

app = get_server(TCPServer, ('localhost', 0), RequestHandler)

route.bind_routes(app)



import pytest
from serial import Serial, serial_for_url
from umodbus.server.serial.rtu import RTUServer, get_char_size


@pytest.fixture
def rtu_server():
    return RTUServer()


def test_get_char_size():
    """ Test if correct char size is calculated. """
    assert get_char_size(11) == 1
    assert get_char_size(19201) == 0.0005


def test_rtu_server_create_response_adu(rtu_server):
    assert rtu_server.create_response_adu({'unit_id': 1}, b'') == b'\x01~\x80'


def test_rtu_server_serial_port(rtu_server):
    """" Test if RTUServer.serial_port sets correct timeout and
    inter_byte_timeout. """
    serial_port = Serial(baudrate=19201)
    rtu_server.serial_port = serial_port

    assert rtu_server.serial_port.timeout == 0.00175
    assert rtu_server.serial_port.inter_byte_timeout == 0.00075


def test_rtu_server_send_empty_message(rtu_server):
    rtu_server.serial_port = serial_for_url('loop://')
    rtu_server.serial_port.write(b'')

    with pytest.raises(ValueError):
        rtu_server.serve_once()



import pytest
import struct
from functools import partial

from ..validators import validate_response_mbap

from umodbus.client.serial import rtu
from umodbus.client.serial.redundancy_check import (get_crc, validate_crc,
                                                    add_crc, CRCError)


def test_no_response_for_request_with_invalid_crc(rtu_server):
    """ Test if server doesn't respond on a request with an invalid CRC. """
    pdu = rtu.read_coils(1, 9, 2)
    adu = struct.pack('>B', 1) + pdu + struct.pack('>BB', 0, 0)

    rtu_server.serial_port.write(adu)

    with pytest.raises(CRCError):
        rtu_server.serve_once()


@pytest.mark.parametrize('function_code, quantity', [
    (1, 0),
    (2, 0),
    (3, 0),
    (4, 0),
    (1, 0x07D0 + 1),
    (2, 0x07D0 + 1),
    (3, 0x007D + 1),
    (4, 0x007D + 1),
])
def test_request_returning_invalid_data_value_error(rtu_server, function_code,
                                                    quantity):
    """ Validate response PDU of request returning excepetion response with
    error code 3.
    """
    starting_address = 0
    slave_id = 1
    adu = add_crc(struct.pack('>BBHH', slave_id, function_code,
                              starting_address, quantity))

    rtu_server.serial_port.write(adu)
    rtu_server.serve_once()
    resp = rtu_server.serial_port.read(rtu_server.serial_port.in_waiting)

    validate_crc(resp)
    assert struct.unpack('>BB', resp[1:-2]) == (0x80 + function_code, 3)


@pytest.mark.parametrize('function', [
    (partial(rtu.read_coils, 1, 9, 2)),
    (partial(rtu.read_discrete_inputs, 1, 9, 2)),
    (partial(rtu.read_holding_registers, 1, 9, 2)),
    (partial(rtu.read_input_registers, 1, 9, 2)),
    (partial(rtu.write_single_coil, 1, 11, 0)),
    (partial(rtu.write_single_register, 1, 11, 1337)),
    (partial(rtu.write_multiple_coils, 1, 9, [1, 1])),
    (partial(rtu.write_multiple_registers, 1, 9, [1337, 15])),
])
def test_request_returning_invalid_data_address_error(rtu_server, function):
    """ Validate response PDU of request returning excepetion response with
    error code 2.
    """
    adu = function()

    function_code = struct.unpack('>B', adu[1:2])[0]

    rtu_server.serial_port.write(adu)
    rtu_server.serve_once()
    resp = rtu_server.serial_port.read(rtu_server.serial_port.in_waiting)

    validate_crc(resp)
    assert struct.unpack('>BB', resp[1:-2]) == (0x80 + function_code, 2)


@pytest.mark.parametrize('function', [
    (partial(rtu.read_coils, 1, 666, 1)),
    (partial(rtu.read_discrete_inputs, 1, 666, 1)),
    (partial(rtu.read_holding_registers, 1, 666, 1)),
    (partial(rtu.read_input_registers, 1, 666, 1)),
    (partial(rtu.write_single_coil, 1, 666, 0)),
    (partial(rtu.write_single_register, 1, 666, 1337)),
    (partial(rtu.write_multiple_coils, 1, 666, [1])),
    (partial(rtu.write_multiple_registers, 1, 666, [1337])),
])
def test_request_returning_server_device_failure_error(rtu_server, function):
    """ Validate response PDU of request returning excepetion response with
    error code 4.
    """
    adu = function()

    function_code = struct.unpack('>B', adu[1:2])[0]

    rtu_server.serial_port.write(adu)
    rtu_server.serve_once()
    resp = rtu_server.serial_port.read(rtu_server.serial_port.in_waiting)

    validate_crc(resp)
    assert struct.unpack('>BB', resp[1:-2]) == (0x80 + function_code, 4)



import struct


def validate_transaction_id(request_mbap, response):
    """ Check if Transaction id in request and response is equal. """
    assert struct.unpack('>H', request_mbap[:2]) == \
        struct.unpack('>H', response[:2])


def validate_protocol_id(request_mbap, response):
    """ Check if Protocol id in request and response is equal. """
    assert struct.unpack('>H', request_mbap[2:4]) == \
        struct.unpack('>H', response[2:4])


def validate_length(response):
    """ Check if Length field contains actual length of response. """
    assert struct.unpack('>H', response[4:6])[0] == len(response[6:])


def validate_unit_id(request_mbap, response):
    """ Check if Unit id in request and response is equal. """
    assert struct.unpack('>B', request_mbap[6:7]) == \
        struct.unpack('>B', response[6:7])


def validate_response_mbap(request_mbap, response):
    """ Validate if fields in response MBAP contain correct values. """
    validate_transaction_id(request_mbap, response)
    validate_protocol_id(request_mbap, response)
    validate_length(response)
    validate_unit_id(request_mbap, response)


def validate_function_code(request, response):
    """ Validate if Function code in request and response equal. """
    assert struct.unpack('>B', request[7:8])[0] == \
        struct.unpack('>B', response[7:8])[0]


def validate_single_bit_value_byte_count(request, response):
    """ Check of byte count field contains actual byte count and if byte count
    matches with the amount of requests quantity.
    """
    byte_count = struct.unpack('>B', response[8:9])[0]

    quantity = struct.unpack('>H', request[-2:])[0]
    expected_byte_count = quantity // 8

    if quantity % 8 != 0:
        expected_byte_count = (quantity // 8) + 1

    assert byte_count == len(response[9:])
    assert byte_count == expected_byte_count


def validate_multi_bit_value_byte_count(request, response):
    """ Check of byte count field contains actual byte count and if byte count
    matches with the amount of requests quantity.
    """
    byte_count = struct.unpack('>B', response[8:9])[0]

    quantity = struct.unpack('>H', request[-2:])[0]
    expected_byte_count = quantity * 2

    assert byte_count == len(response[9:])
    assert byte_count == expected_byte_count



import pytest
from serial import serial_for_url

from umodbus.client.serial.rtu import send_message, read_coils


def test_send_message_with_timeout():
    """ Test if TimoutError is raised when serial port doesn't receive enough
    data.
    """
    s = serial_for_url('loop://', timeout=0)
    # As we are using a loop, the sent request will be read back as response.
    # To test timeout use a request that needs more bytes for the response.
    message = read_coils(slave_id=0, starting_address=1, quantity=40)

    with pytest.raises(ValueError):
        send_message(message, s)



#!/usr/bin/env python
# scripts/examples/simple_tcp_client.py
import socket

from umodbus import conf
from umodbus.client import tcp

# Enable values to be signed (default is False).
conf.SIGNED_VALUES = True

sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.connect(('localhost', 502))

# Returns a message or Application Data Unit (ADU) specific for doing
# Modbus TCP/IP.
message = tcp.write_multiple_coils(slave_id=1, starting_address=1, values=[1, 0, 1, 1])

# Response depends on Modbus function code. This particular returns the
# amount of coils written, in this case it is.
response = tcp.send_message(message, sock)

sock.close()



"""
.. note:: This section is based on `MODBUS over Serial Line Specification and
    Implementation Guide V1.02`_.

The ADU for Modbus RTU messages differs from Modbus TCP/IP messages. Messages
send over RTU don't have a MBAP header, instead they have an Address field.
This field contains the slave id.  A CRC is appended to the message. Below all
parts of a Modbus RTU message are listed together with their byte size:

+---------------+-----------------+
| **Component** | **Size** (bytes)|
+---------------+-----------------+
| Address field | 1               |
+---------------+-----------------+
| PDU           | N               |
+---------------+-----------------+
| CRC           | 2               |
+---------------+-----------------+

The CRC is calculated from the Address field and the PDU.

Below you see a hexadecimal presentation of request over RTU with Modbus
function code 1. It requests data of slave with 1, starting at coil 100, for
the length of 3 coils:

..
    Note: the backslash in the bytes below are escaped using an extra back
    slash. Without escaping the bytes aren't printed correctly in the HTML
    output of this docs.

    To work with the bytes in Python you need to remove the escape sequences.
    `b'\\x01\\x00d` -> `b\x01\x00d`

.. code-block:: python

    >>> # Read coils, starting from coil 100 for the length of 3 coils.
    >>> adu = b'\\x01\\x01\\x00d\\x00\\x03=\\xd4'

The lenght of this ADU is 8 bytes::

    >>> len(adu)
    8

"""
import struct

from umodbus.client.serial.redundancy_check import get_crc, validate_crc
from umodbus.functions import (create_function_from_response_pdu,
                               expected_response_pdu_size_from_request_pdu,
                               pdu_to_function_code_or_raise_error, ReadCoils,
                               ReadDiscreteInputs, ReadHoldingRegisters,
                               ReadInputRegisters, WriteSingleCoil,
                               WriteSingleRegister, WriteMultipleCoils,
                               WriteMultipleRegisters)
from umodbus.utils import recv_exactly


def _create_request_adu(slave_id, req_pdu):
    """ Return request ADU for Modbus RTU.

    :param slave_id: Slave id.
    :param req_pdu: Byte array with PDU.
    :return: Byte array with ADU.
    """
    first_part_adu = struct.pack('>B', slave_id) + req_pdu

    return first_part_adu + get_crc(first_part_adu)


def read_coils(slave_id, starting_address, quantity):
    """ Return ADU for Modbus function code 01: Read Coils.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = ReadCoils()
    function.starting_address = starting_address
    function.quantity = quantity

    return _create_request_adu(slave_id, function.request_pdu)


def read_discrete_inputs(slave_id, starting_address, quantity):
    """ Return ADU for Modbus function code 02: Read Discrete Inputs.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = ReadDiscreteInputs()
    function.starting_address = starting_address
    function.quantity = quantity

    return _create_request_adu(slave_id, function.request_pdu)


def read_holding_registers(slave_id, starting_address, quantity):
    """ Return ADU for Modbus function code 03: Read Holding Registers.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = ReadHoldingRegisters()
    function.starting_address = starting_address
    function.quantity = quantity

    return _create_request_adu(slave_id, function.request_pdu)


def read_input_registers(slave_id, starting_address, quantity):
    """ Return ADU for Modbus function code 04: Read Input Registers.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = ReadInputRegisters()
    function.starting_address = starting_address
    function.quantity = quantity

    return _create_request_adu(slave_id, function.request_pdu)


def write_single_coil(slave_id, address, value):
    """ Return ADU for Modbus function code 05: Write Single Coil.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = WriteSingleCoil()
    function.address = address
    function.value = value

    return _create_request_adu(slave_id, function.request_pdu)


def write_single_register(slave_id, address, value):
    """ Return ADU for Modbus function code 06: Write Single Register.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = WriteSingleRegister()
    function.address = address
    function.value = value

    return _create_request_adu(slave_id, function.request_pdu)


def write_multiple_coils(slave_id, starting_address, values):
    """ Return ADU for Modbus function code 15: Write Multiple Coils.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = WriteMultipleCoils()
    function.starting_address = starting_address
    function.values = values

    return _create_request_adu(slave_id, function.request_pdu)


def write_multiple_registers(slave_id, starting_address, values):
    """ Return ADU for Modbus function code 16: Write Multiple Registers.

    :param slave_id: Number of slave.
    :return: Byte array with ADU.
    """
    function = WriteMultipleRegisters()
    function.starting_address = starting_address
    function.values = values

    return _create_request_adu(slave_id, function.request_pdu)


def parse_response_adu(resp_adu, req_adu=None):
    """ Parse response ADU and return response data. Some functions require
    request ADU to fully understand request ADU.

    :param resp_adu: Resonse ADU.
    :param req_adu: Request ADU, default None.
    :return: Response data.
    """
    resp_pdu = resp_adu[1:-2]
    validate_crc(resp_adu)

    req_pdu = None

    if req_adu is not None:
        req_pdu = req_adu[1:-2]

    function = create_function_from_response_pdu(resp_pdu, req_pdu)

    return function.data


def raise_for_exception_adu(resp_adu):
    """ Check a response ADU for error

    :param resp_adu: Response ADU.
    :raises ModbusError: When a response contains an error code.
    """
    resp_pdu = resp_adu[1:-2]
    pdu_to_function_code_or_raise_error(resp_pdu)


def send_message(adu, serial_port):
    """ Send ADU over serial to to server and return parsed response.

    :param adu: Request ADU.
    :param sock: Serial port instance.
    :return: Parsed response from server.
    """
    serial_port.write(adu)
    serial_port.flush()

    # Check exception ADU (which is shorter than all other responses) first.
    exception_adu_size = 5
    response_error_adu = recv_exactly(serial_port.read, exception_adu_size)
    raise_for_exception_adu(response_error_adu)

    expected_response_size = \
        expected_response_pdu_size_from_request_pdu(adu[1:-2]) + 3
    response_remainder = recv_exactly(
        serial_port.read, expected_response_size - exception_adu_size)

    return parse_response_adu(response_error_adu + response_remainder, adu)



#!/usr/bin/env python
# scripts/example/simple_rtu_client.py
import fcntl
import struct
from serial import Serial, PARITY_NONE

from umodbus.client.serial import rtu


def get_serial_port():
    """ Return serial.Serial instance, ready to use for RS485."""
    port = Serial(port='/dev/ttyS1', baudrate=9600, parity=PARITY_NONE,
                  stopbits=1, bytesize=8, timeout=1)

    fh = port.fileno()

    # A struct with configuration for serial port.
    serial_rs485 = struct.pack('hhhhhhhh', 1, 0, 0, 0, 0, 0, 0, 0)
    fcntl.ioctl(fh, 0x542F, serial_rs485)

    return port

serial_port = get_serial_port()

# Returns a message or Application Data Unit (ADU) specific for doing
# Modbus RTU.
message = rtu.write_multiple_coils(slave_id=1, address=1, values=[1, 0, 1, 1])

# Response depends on Modbus function code. This particular returns the
# amount of coils written, in this case it is.
response = rtu.send_message(message, serial_port)

serial_port.close()



""" The module umodbus.server is mainly covered through system tests. Only
those parts which can't be tested by system tests should be tested using
unit tests.
"""
import struct
import pytest

from umodbus.exceptions import ServerDeviceFailureError
from umodbus.client.tcp import read_coils
from umodbus.server.tcp import RequestHandler


@pytest.fixture
def meta_data():
    return {
        'transaction_id': 1337,
        'protocol_id': 0,
        'length': 1,
        'unit_id': 5,
    }


@pytest.fixture
def mbap_header():
    transaction_id = 1337
    protocol_id = 0
    length = 1
    unit_id = 5

    return struct.pack('>HHHB', transaction_id, protocol_id, length, unit_id)


@pytest.fixture
def request_handler(monkeypatch):
    # handle() is called when after creating a RequestHandler. This
    # causes an error because it wants to read from a socket. Mock it out so
    # this error doesn't occur.
    monkeypatch.setattr(RequestHandler, 'handle', lambda _: None)
    return RequestHandler(None, None, None)


def test_handle_raising_exception():
    """ Test tests RequestHandler.handle() which is called when an instance
    of RequestHandler is created. This method should reraise exception if one
    occurs.
    """
    with pytest.raises(AttributeError):
        RequestHandler(None, None, None)


def test_request_handler_get_meta_data(request_handler, mbap_header,
                                       meta_data):
    assert request_handler.get_meta_data(mbap_header) == meta_data


def test_request_handler_get_meta_data_raising_error(request_handler):
    with pytest.raises(ServerDeviceFailureError):
        request_handler.get_meta_data(b'')


def def_test_get_request_pdu(request_handler, mbap_header):
    pdu = read_coils(1, 1, 1)
    assert request_handler.get_request_pdu(mbap_header + pdu) == pdu


def test_response_adu(request_handler, mbap_header, meta_data):
    assert len(request_handler.create_response_adu(meta_data, b'')) == 7



#!/usr/bin/env python
"""
uModbus is a pure Python implementation of the Modbus protocol with support
for Python 2.7, 3.4, 3.5, 3.6, 3.7 and 3.8.

"""
import os
from setuptools import setup

cwd = os.path.dirname(os.path.abspath(__name__))

long_description = open(os.path.join(cwd, 'README.rst'), 'r').read()

setup(name='uModbus',
      version='1.0.4',
      author='Auke Willem Oosterhoff',
      author_email='a.oosterhoff@climotion.com',
      description='Implementation of the Modbus protocol in pure Python.',
      url='https://github.com/AdvancedClimateSystems/umodbus/',
      long_description=long_description,
      license='MPL',
      packages=[
          'umodbus',
          'umodbus.client',
          'umodbus.client.serial',
          'umodbus.server',
          'umodbus.server.serial',
      ],
      install_requires=[
          'pyserial~=3.4',
      ],
      classifiers=[
          'Development Status :: 6 - Mature',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Topic :: Software Development :: Embedded Systems',
      ])



from __future__ import division
import struct
from binascii import hexlify

from umodbus import log
from umodbus.server.serial import AbstractSerialServer
from umodbus.client.serial.redundancy_check import get_crc, validate_crc


def get_char_size(baudrate):
    """ Get the size of 1 character in seconds.

    From the implementation guide:

        "The implementation of RTU reception driver may imply the management of
        a lot of interruptions due to the t 1.5  and t 3.5  timers. With high

        communication baud rates, this leads to a heavy CPU load. Consequently
        these two timers must be strictly respected when the baud rate is equal
        or lower than 19200 Bps. For baud rates greater than 19200 Bps, fixed
        values for the 2 timers should be used:  it is recommended to use a
        value of 750us for the inter-character time-out (t 1.5) and a value of
        1.750ms for inter-frame delay (t 3.5)."
    """
    if baudrate <= 19200:
        # One frame is 11 bits.
        return 11 / baudrate

    # 750 us / 1.5 = 500 us or 0.0005 s.
    return 0.0005


class RTUServer(AbstractSerialServer):
    @property
    def serial_port(self):
        return self._serial_port

    @serial_port.setter
    def serial_port(self, serial_port):
        """ Set timeouts on serial port based on baudrate to detect frames. """
        char_size = get_char_size(serial_port.baudrate)

        # See docstring of get_char_size() for meaning of constants below.
        serial_port.inter_byte_timeout = 1.5 * char_size
        serial_port.timeout = 3.5 * char_size
        self._serial_port = serial_port

    def serve_once(self):
        """ Listen and handle 1 request. """
        # 256 is the maximum size of a Modbus RTU frame.
        request_adu = self.serial_port.read(256)
        log.debug('<-- {0}'.format(hexlify(request_adu)))

        if len(request_adu) == 0:
            raise ValueError

        response_adu = self.process(request_adu)
        self.respond(response_adu)

    def process(self, request_adu):
        """ Process request ADU and return response.

        :param request_adu: A bytearray containing the ADU request.
        :return: A bytearray containing the response of the ADU request.
        """
        validate_crc(request_adu)
        return super(RTUServer, self).process(request_adu)

    def create_response_adu(self, meta_data, response_pdu):
        """ Build response ADU from meta data and response PDU and return it.

        :param meta_data: A dict with meta data.
        :param request_pdu: A bytearray containing request PDU.
        :return: A bytearray containing request ADU.
        """
        first_part_adu = struct.pack('>B', meta_data['unit_id']) + response_pdu
        return first_part_adu + get_crc(first_part_adu)



import pytest

from umodbus import conf
from umodbus.client import tcp


@pytest.fixture(scope='module', autouse=True)
def enable_signed_values(request):
    """ Use signed values when running tests it this module. """
    tmp = conf.SIGNED_VALUES
    conf.SIGNED_VALUES = True

    def fin():
        conf.SIGNED_VALUES = tmp

    request.addfinalizer(fin)


@pytest.mark.parametrize('function', [
    tcp.read_coils,
    tcp.read_discrete_inputs,
])
def test_response_on_single_bit_value_read_requests(sock, function):
    """ Validate response of a succesful Read Coils or Read Discrete Inputs
    request.
    """
    slave_id, starting_address, quantity = (1, 0, 10)
    req_adu = function(slave_id, starting_address, quantity)

    assert tcp.send_message(req_adu, sock) == [0, 1, 0, 1, 0, 1, 0, 1, 0,  1]


@pytest.mark.parametrize('function', [
    tcp.read_holding_registers,
    tcp.read_input_registers,
])
def test_response_on_multi_bit_value_read_requests(sock, function):
    """ Validate response of a succesful Read Holding Registers or Read
    Input Registers request.
    """
    slave_id, starting_address, quantity = (1, 0, 10)
    req_adu = function(slave_id, starting_address, quantity)

    assert tcp.send_message(req_adu, sock) ==\
        [0, -1, -2, -3, -4, -5, -6, -7, -8, -9]


@pytest.mark.parametrize('function, value', [
    (tcp.write_single_coil, 1),
    (tcp.write_single_register, -1337),
])
def test_response_single_value_write_request(sock, function, value):
    """ Validate responde of succesful Read Single Coil and Read Single
    Register request.
    """
    slave_id, starting_address, value = (1, 0, value)
    req_adu = function(slave_id, starting_address, value)

    assert tcp.send_message(req_adu, sock) == value


@pytest.mark.parametrize('function, values', [
    (tcp.write_multiple_coils, [1, 1]),
    (tcp.write_multiple_registers, [1337, 15]),
])
def test_response_multi_value_write_request(sock, function, values):
    """ Validate response of succesful Write Multiple Coils and Write Multiple
    Registers request.

    Both requests write 2 values, starting address is 0.
    """
    slave_id, starting_address = (1, 0)
    req_adu = function(slave_id, starting_address, values)

    assert tcp.send_message(req_adu, sock) == 2



import struct
import pytest

from umodbus.client.serial.redundancy_check import (get_crc, validate_crc,
                                                    CRCError)


def test_get_crc():
    """ Test if correct CRC is calculated. """
    # Values are equal to those used in example in MODBUS over serial line
    # specification and implementation guide V1.02, chapter 6.2.2.
    assert struct.unpack('<H', get_crc(b'\x02\x07')) ==\
        struct.unpack('<H', b'\x41\x12')


def test_validate_valid_crc():
    """" Method should not raise assertion error. """
    validate_crc(b'\x00\x01\x02\xf1\x91')


def test_validate_invalid_crc():
    """" Method should raise assertion error. """
    with pytest.raises(CRCError):
        validate_crc(b'\x01\x02\x07')



import pytest

from umodbus.route import DataRule


endpoint = lambda slave_id, function_code, address: 0


def test_basic_route():
    rule = DataRule(endpoint, slave_ids=[1], function_codes=[1], addresses=[1])
    assert rule.match(slave_id=1, function_code=1, address=1)
    assert not rule.match(slave_id=0, function_code=1, address=1)
    assert not rule.match(slave_id=1, function_code=0, address=1)
    assert not rule.match(slave_id=1, function_code=1, address=0)


def test_other_iterables():
    # Other iterable types should work, not just lists
    rule = DataRule(endpoint,
                    slave_ids=set([1]), function_codes=[1], addresses=[1])
    assert rule.match(slave_id=1, function_code=1, address=1)


def test_wildcard_slave_id():
    rule = DataRule(endpoint, slave_ids=None, function_codes=[1], addresses=[1])
    assert rule.match(slave_id=1, function_code=1, address=1)


def test_wildcard_function_code():
    rule = DataRule(endpoint, slave_ids=[1], function_codes=None, addresses=[1])
    assert rule.match(slave_id=1, function_code=1, address=1)


def test_wildcard_address():
    rule = DataRule(endpoint, slave_ids=[1], function_codes=[1], addresses=None)
    assert rule.match(slave_id=1, function_code=1, address=1)



class TestConfig:
    def test_defaults(self, config):
        """ Test whether defaults configuration values are correct. """
        assert config.SINGLE_BIT_VALUE_FORMAT_CHARACTER == 'B'
        assert config.MULTI_BIT_VALUE_FORMAT_CHARACTER == 'H'
        assert not config.SIGNED_VALUES

    def test_multi_bit_value_signed(self, config):
        """  Test if MULTI_BIT_VALUE_FORMAT_CHARACTER changes when setting
        signedness.
        """
        assert config.MULTI_BIT_VALUE_FORMAT_CHARACTER == 'H'
        config.SIGNED_VALUES = True
        assert config.MULTI_BIT_VALUE_FORMAT_CHARACTER == 'h'
